/*
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.network.synchronization.protocol;

import java.util.List;

import com.topcoder.network.synchronization.bean.Message;
import com.topcoder.network.synchronization.bean.PeerGroupInfo;
import com.topcoder.network.synchronization.bean.PeerInfo;
import com.topcoder.network.synchronization.exception.DistributedProtocolException;



/**
 * It is the main interface in this component. It provides functionalities to send synchronized messages to the peer groups.
 * That means all the peers in the peer group will receive the messages in the same order.
 * 
 * It also provides method to re-synchronize the peer with the other peers in the peer group. A peer may be unsynchronized for
 * any reasons, bad network, too long time to process the message, etc.
 * 
 * It also provides methods to set the listeners to process the message and handle errors.  
 * 
 *   
 * 
 * @author TCSDESIGNER
 * @version 1.0
 */
public interface DistributedSynchronizationProtocol {
    
    /**
     * Return the id of the protocol.
     * 
     * @return the id of the protocol
     */
    String getProtocolId();
    
    /**
     * Start the protocol.
     * 
     * @throws DistributedProtocolException if fails to start the protocol
     */
    void start() throws DistributedProtocolException;
    
    /**
     * Stop the protocol.
     * 
     * @throws DistributedProtocolException if fails to stop the protocol
     */
    void stop() throws DistributedProtocolException;
   
    /**
     * Add APPLICATION_MESSAGE listener to this service.
     * 
     * @param listener APPLICATION_MESSAGE listener to add
     */
    void addApplicationMessageListener(MessageListener listener);
    
    /**
     * Remove APPLICATION_MESSAGE listener to this service.
     * @param listener the APPLICATION_MESSAGE listener to remove
     */
    void removeApplicationMessageListener(MessageListener listener);
    
    /**
     * Return the first APPLICATION_MESSAGE listener in this service.
     * @return the first APPLICATION_MESSAGE listener in this service.
     * 
     * 
     */
    MessageListener getApplicationMessageListener();
    
    
    /**
     * Set the listener to handle exceptions.
     * 
     * @param listener the listener to handle exceptions
     */
    void setErrorHandlingListener(ErrorHandlingListener listener);
    
    /**
     * Return the error handling listener to handle exceptions
     * 
     * @return the error handling listener to handle exceptions
     */
    ErrorHandlingListener getErrorHandlingListener();
    
    /**
     * Get all the peer groups(a list of PeerGroupInfo object) in the network. 
     * 
     * @param timeout the time needs to wait to retrieve peer groups 
     * @return all the peer groups, a list of PeerGroupInfo object
     * @throws DistributedProtocolException if fails to get the peer groups from the network
     * @throws IllegalArgumentException if timeout is not positive
     */
    List getAllPeerGroups(int timeout)throws DistributedProtocolException;
    
    /**
     * Join the peer group specified in the groupInfo.
     * 
     * @param groupInfo
     * @throws DistributedProtocolException
     */
    void joinPeerGroup(PeerGroupInfo groupInfo) throws DistributedProtocolException;
    
    /**
     * Leave the current peer group.
     * 
     * @throws DistributedProtocolException
     */
    void leavePeerGroup() throws DistributedProtocolException;
    
    /**
     * Move this peer from current peer group to the specified peer group.
     * 
     * Note: first leave the current peer group, then join the specified peer group
     * 
     * @param groupInfo the peer group to move
     * @throws DistributedProtocolException if fails to move this peer to the specified peer group
     */
    void MoveToPeerGroup(PeerGroupInfo groupInfo) throws DistributedProtocolException;
    
    /**
     * Send the message to all the peers in the current peer group.
     * 
     * @param message the message to send  
     * @throws DistributedProtocolException if fails to send the message
     */
    void sentMessage(Message message) throws DistributedProtocolException;
       
    /**
     * Return the PeerGroupService in this protocol.
     * @return the PeerGroupService in this protocol.
     */
    PeerGroupService getPeerGroupService();
    
    /**
     * Return the information of the current peer group joined.
     *  
     * @return the information of the current peer group joined
     */
    PeerGroupInfo getPeerGroupInfo();
    
    /**
     * Return the information of this peer. 
     * 
     * @return the information of this peer
     */
    PeerInfo getPeerInfo();
}
